@@ -1,8 +1,8 @@
-;;; yatexmth.el --- YaTeX math-mode-specific functions
+;;; yatexmth.el --- YaTeX math-mode-specific functions -*- coding: sjis -*-
 ;;; 
-;;; (c)1993-2013 by HIROSE Yuuji [yuuji@yatex.org]
-;;; Last modified Mon Apr  1 22:44:22 2013 on firestorm
-;;; $Id: yatexmth.el,v 1.77 2013/04/01 13:53:45 yuuji Rel $
+;;; (c)1993-2017 by HIROSE Yuuji [yuuji@yatex.org]
+;;; Last modified Sat Feb 24 16:40:22 2018 on firestorm
+;;; $Id: yatexmth.el,v a58a35eac93f 2018-07-31 08:21 +0900 yuuji $
 
 ;;; Commentary:
 ;;; [Customization guide]
@@ -310,6 +310,8 @@
    ("[]"	"Box"		"[]")
    ("no"	"notag"		"\\notag")
    (":"		"colon"		":")
+   (".'."	"therefore"	(".'." ""))
+   ("'.'"	"because"	("'.'" ""))
    ("Diamond"	"Diamond"	"/\\\n\\/")
    ("3"		"triangle"	"/\\\n~~")
    ("C"		"clubsuit"	" o\no+o\n |")
@@ -319,6 +321,7 @@
    ("mi"	"mathit"	"\\mathit{}")
    ("mr"	"mathrm"	"\\mathrm{}")
    ("mb"	"mathbf"	"\\mathbf{}")
+   ("mB"	"mathbb"	"\\mathbb{}")
    ("mt"	"mathtt"	"\\mathtt{}")
    ("ms"	"mathsf"	"\\mathsf{}")
    ("mc"	"mathcal"	"\\mathcal{}")
@@ -344,53 +347,64 @@
 ;;)
 
 (defvar YaTeX-greek-key-alist-default
-  '(
-    ("a"	"alpha"		("a" ""))
-    ("b"	"beta"		("|>\n|>\n|" ""))
-    ("g"	"gamma"		("~r" ""))
-    ("G"	"Gamma"		("|~" ""))
-    ("d"	"delta"		("<~\n<>" ""))
-    ("D"	"Delta"		("/\\\n~~" ""))
-    ("e"	"epsilon"	"<\n<~")
-    ("e-"	"varepsilon"	("(\n(~" ""))
-    ("z"	"zeta"		("(~\n >" ""))
-    ("et"	"eta"		("n\n/" ""))
-    ("th"	"theta"		("8" ""))
-    ("Th"	"Theta"		("(8)" ""))
-    ("th-"	"vartheta"	("-8" "-"))
-    ("i"	"iota"		("i\n\\_/" ""))
-    ("k"	"kappa"		("k" ""))
-    ("l"	"lambda"	("\\n/\\" ""))
-    ("L"	"Lambda"	("/\\" ""))
-    ("m"	"mu"		(" u_\n/" ""))
-    ("n"	"nu"		("|/" ""))
-    ("x"	"xi"		("E\n >" ""))
-    ("X"	"Xi"		("---\n -\n---" ""))
-    ("p"	"pi"		("__\n)(" ""))
-    ("P"	"Pi"		("__\n||" ""))
-    ("p-"	"varpi"		("_\nw" "__\n"))
-    ("r"	"rho"		("/O" ""))
-    ("r-"	"varrho"	("/O\n~~" "\n~~"))
-    ("s"	"sigma"		("o~" ""))
-    ("S"	"Sigma"		("\\-+\n >\n/-+" ""))
-    ("s-"	"varsigma"	"(~~ \n>")
-    ("t"	"tau"		("__\n(" ""))
-    ("u"	"upsilon"	("~v" ""))
-    ("y"	"upsilon"	("~v" ""))
-    ("U"	"Upsilon"	("~Y~" ""))
-    ("Y"	"Upsilon"	("~Y~" ""))
-    ("ph"	"phi"		("  /\n(/)\n/" ""))
-    ("Ph"	"Phi"		(" _\n(|)\n ~" ""))
-    ("ph-"	"varphi"	"\\O\n|")
-    ("c"	"chi"		("x" ""))
-    ("ps"	"psi"		("\\|/\\n |" ""))
-    ("Ps"	"Psi"		(" ~\n\\|/\\n |" ""))
-    ("o"	"omega"		("w" ""))
-    ("w"	"omega"		("w" ""))
-    ("O"	"Omega"		("(~)\n~ ~" ""))
-    ("W"	"Omega"		("(~)\n~ ~" ""))
-    ("f" "foo")
-    )
+  (append
+   '(("a"	"alpha"		("a" ""))
+     ("b"	"beta"		("|>\n|>\n|" ""))
+     ("g"	"gamma"		("~r" ""))
+     ("G"	"Gamma"		("|~" ""))
+     ("d"	"delta"		("<~\n<>" ""))
+     ("D"	"Delta"		("/\\\n~~" ""))
+     ("e"	"epsilon"	"<\n<~")
+     ("e-"	"varepsilon"	("(\n(~" "_"))
+     ("z"	"zeta"		("(~\n >" ""))
+     ("et"	"eta"		("n\n/" ""))
+     ("th"	"theta"		("8" ""))
+     ("Th"	"Theta"		("(8)" ""))
+     ("th-"	"vartheta"	("-8" "_"))
+     ("i"	"iota"		("i\n\\_/" ""))
+     ("k"	"kappa"		("k" ""))
+     ("l"	"lambda"	("\\n/\\" ""))
+     ("L"	"Lambda"	("/\\" ""))
+     ("m"	"mu"		(" u_\n/" ""))
+     ("n"	"nu"		("|/" ""))
+     ("x"	"xi"		("E\n >" ""))
+     ("X"	"Xi"		("---\n -\n---" ""))
+     ("p"	"pi"		("__\n)(" ""))
+     ("P"	"Pi"		("__\n||" ""))
+     ("p-"	"varpi"		("__\n/(" "_"))
+     ("r"	"rho"		("/O" ""))
+     ("r-"	"varrho"	("/O\n~~" "\n~~"))
+     ("s"	"sigma"		("o~" ""))
+     ("S"	"Sigma"		("\\-+\n >\n/-+" ""))
+     ("s-"	"varsigma"	"/~~ \n /")
+     ("t"	"tau"		("__\n(" ""))
+     ("u"	"upsilon"	("~v" ""))
+     ("y"	"upsilon"	("~v" ""))
+     ("U"	"Upsilon"	("~Y~" ""))
+     ("Y"	"Upsilon"	("~Y~" ""))
+     ("ph"	"phi"		("  /\n(/)\n/" ""))
+     ("Ph"	"Phi"		(" _\n(|)\n ~" ""))
+     ("ph-"	"varphi"	"\\O\n|")
+     ("c"	"chi"		("x" ""))
+     ("ps"	"psi"		("\\|/\\n |" ""))
+     ("Ps"	"Psi"		(" ~\n\\|/\\n |" ""))
+     ("o"	"omega"		("w" ""))
+     ("w"	"omega"		("w" ""))
+     ("O"	"Omega"		("(~)\n~ ~" ""))
+     ("W"	"Omega"		("(~)\n~ ~" ""))
+     ("f" "foo"))
+   (if YaTeX-use-AMS-LaTeX
+       '(
+	 ("G-"	"varGamma"	("/~" "_"))
+	 ("D-"	"varDelta"	("/|\n~~" "_"))
+	 ("Th-"	"varTheta"	("/8/" "_"))
+	 ("L-"	"varLambda"	("/|" "_"))
+	 ("X-"	"varXi"		(" --\n -\n-- " "_"))
+	 ("S-"	"varSigma"	(" \\-+\n >\n/-+" "_"))
+	 ("U-"	"varUpsilon"	("~~Y~" "_"))
+	 ("Ph-"	"varPhi"	("  _\n(|)\n~" "_"))
+	 ("Ps-"	"varPsi"	("  ~\n\\//\\n /" ""))
+	 ("O-"	"varOmega"	("/~/\n~ ~" "_")))))
   "Default LaTeX-math-command alist.")
 
 (defvar YaTeX-greek-key-alist-private nil
@@ -451,7 +465,7 @@
 ;;;
 ;;YaTeX math-mode functions
 ;;;
-;;;###autoload from yatex.el
+;;;###autoload
 (defun YaTeX-toggle-math-mode (&optional arg)
   (interactive "P")
   (or (memq 'YaTeX-math-mode mode-line-format) nil
@@ -520,7 +534,8 @@ This function refers a local variable `source-window' in YaTeX-make-section."
 	      ;;  "equation*" "cases" "flalign" "flalign*"
 	      ;;  "alignat*" "xalignat" "xalignat*" "xxalignat" "xxalignat*"
 	      YaTeX-math-begin-list
-	    )))
+	    )
+	  YaTeX-math-other-env-list))
 	(let*((p (point)) (nest 0) me0 r firstp dollar
 	      (delim (concat YaTeX-sectioning-regexp "\\|^$\\|^\C-l"))
 	      (boundary
@@ -658,7 +673,6 @@ This function refers a local variable `source-window' in YaTeX-make-section."
 	(kill-buffer YaTeX-math-menu-buffer))
       command)))
 
-;
 (defun YaTeX-math-show-image (image &optional exit-char)
   "Momentarily display IMAGE at the beginning of the next line;
 erase it on the next keystroke.  The window is recentered if necessary
@@ -971,9 +985,9 @@ If optional argument JUMPTO-CO is non-nil, goto corresponding parentheses."
 	    (t
 	     (setq size nil lr nil)))
       (while (not newsize)
-	(message (format (concat "Change from %s: "
+	(message (concat "Change from %s: "
 				 "l(big) L(Big) h(bigg) H(Bigg) "
-				 "r(left-right) n(NONE) ( { [") size))
+				 "r(left-right) n(NONE) ( { [") size)
 	(setq char (read-char)
 	      newsize (cond ((char-equal char ?l) "\\big")
 			    ((char-equal char ?L) "\\Big")
